

%% init

clear;

%% Load ARX Model

load('database_myARXModel.mat');

%% Calculation settings

% parameters to calculate

Cpc_list = 1.18 : 0.001 : 1.23;
lambda_list = 100:10:1000;

%%%%%%%%%%%%%%%%

M   = 2.0;           % Mach number
l   = 1.0;           % plate length
h   = 0.002;         % plate thickness

beta = (M*M-1)^0.5;

R0 = 0;              % initial in-plane force (non-dimensional)

lambda_train = 600;  % non-dimensional dynamic pressure used in the training process
DT_train = 0.0005;   % time step used in the training process


%% Get ARX coefficient matrix

anchor = 1;

for i = 1:myARXModel.na
    myARXModel.A(1:myARXModel.n_out,1:myARXModel.n_out,i) = ...
        myARXModel.THETA(1:myARXModel.n_out, anchor:anchor+myARXModel.n_out-1);
    anchor = anchor + myARXModel.n_out;
end
for i = 1:myARXModel.nb
    myARXModel.B(1:myARXModel.n_out,1:myARXModel.n_in,i) = ...
        myARXModel.THETA(1:myARXModel.n_out, anchor:anchor+myARXModel.n_in-1);
    anchor = anchor + myARXModel.n_in;
end


%% Loop for parameters

for count_pc_target = 1:length(Cpc_list)
    Cpc = Cpc_list(count_pc_target);
    for count_lambda = 1:length(lambda_list)
        lambda = lambda_list(count_lambda);

        disp(' ')
        disp(Cpc)
        disp(' ')
        disp(lambda)
        disp(' ')


        %% Get static ARX model coefficient matrix

        % staticARX is the coefficient matrix of static ARX model

        if myARXModel.na > 0
            staticARX = (diag(ones(1,n_out))-sum(myARXModel.A,3)) \ sum(myARXModel.B,3) ;
        else
            staticARX = sum(myARXModel.B,3);
        end        

        %% Solve fixed point position

        e = optimvar('e',myARXModel.n_in,1);

        equ_struc = fcn2optimexpr(@strucEqu, ...
            e, ...
            Cpc, ...
            lambda, ...
            staticARX, ...
            myARXModel.initialValue, ...
            M, l, h, R0);

        equ_solve = equ_struc == 0;

        prob = eqnproblem;
        prob.Equations.equ = equ_solve;

        % show(prob)

        e0.e = zeros(myARXModel.n_in,1); % initial value for iteration

        sol = solve(prob,e0);

        % sol.e

        optResidual = sum(abs(strucEqu(sol.e, ...
            Cpc, ...
            lambda, ...
            staticARX, ...
            myARXModel.initialValue, ...
            M, l, h, R0)));

        if optResidual > 1e-6
            error('get a wrong static deformation!')
        end



        %% Get state space model coefficient matrix

        nab = max(myARXModel.na,myARXModel.nb-1);
        stateSpaceModel.A = zeros(nab*myARXModel.n_out, nab*myARXModel.n_out);
        stateSpaceModel.B = zeros(nab*myARXModel.n_out,     myARXModel.n_in);
        stateSpaceModel.C = zeros(    myARXModel.n_out, nab*myARXModel.n_out);
        stateSpaceModel.D = zeros(    myARXModel.n_out,     myARXModel.n_in);

        % get A
        if myARXModel.na > 0
            temp = [];
            for i = 1:myARXModel.na
                temp = [temp;
                    myARXModel.A(:,:,i)];
            end
            stateSpaceModel.A(1:myARXModel.na*myARXModel.n_out, ...
                1:myARXModel.n_out) = temp;
        end
        stateSpaceModel.A(1:(nab-1)*myARXModel.n_out, ...
            myARXModel.n_out+(1:(nab-1)*myARXModel.n_out)) = ...
            diag(ones(1,(nab-1)*myARXModel.n_out));

        % get B
        if myARXModel.na > 0
            temp = [];
            for i = 1:myARXModel.na
                temp = [temp;
                    myARXModel.A(:,:,i)*myARXModel.B(:,:,0+1)];
            end
            stateSpaceModel.B(1:myARXModel.na*myARXModel.n_out,:) = ...
                stateSpaceModel.B(1:myARXModel.na*myARXModel.n_out,:) + temp;
        end
        temp = [];
        for i = 1:myARXModel.nb-1
            temp = [temp;
                myARXModel.B(:,:,i+1)];
        end
        stateSpaceModel.B(1:(myARXModel.nb-1)*myARXModel.n_out,:) = ...
            stateSpaceModel.B(1:(myARXModel.nb-1)*myARXModel.n_out,:) + temp;
        
        % get C
        stateSpaceModel.C(:,1:myARXModel.n_out) = diag(ones(1,myARXModel.n_out));
        
        % get D
        stateSpaceModel.D(:,:) = myARXModel.B(:,:,0+1);


        %% Convert to continuous-time

        timefactor = (lambda/lambda_train)^0.5;
        DT = DT_train/timefactor;


        stateSpaceModel_d = ss(stateSpaceModel.A,stateSpaceModel.B,stateSpaceModel.C,stateSpaceModel.D,DT);
        stateSpaceModel_c = d2c(stateSpaceModel_d,'tustin');

        % size(stateSpaceModel_c)

        aero.A = get(stateSpaceModel_c,'A');  % A of continuous-time state space model 
        aero.B = get(stateSpaceModel_c,'B');  % B of continuous-time state space model 
        aero.C = get(stateSpaceModel_c,'C');  % C of continuous-time state space model 
        aero.D = get(stateSpaceModel_c,'D');  % D of continuous-time state space model 



        %% Get Jacobian matrix

        n_aero = size(aero.A,1);

        n_equCombine = myARXModel.n_in*2 + n_aero;

        Jacobi = zeros(n_equCombine,n_equCombine);

        for i = 1:myARXModel.n_in
            i_local = i;
            % ## 1 #########
            for j = (1:myARXModel.n_in)
                % j_local = j;
                % reamin 0
            end
            % ## 2 #########
            for j = (1:myARXModel.n_in) + myARXModel.n_in
                j_local = j-myARXModel.n_in;
                if i_local == j_local
                    Jacobi(i,j) = 1;
                end
            end
            % ## 3 #########
            for j = (1:n_aero) + 2*myARXModel.n_in
                % j_local = j-2*n_in;
                % reamin 0
            end
        end


        for i = (1:myARXModel.n_in) + myARXModel.n_in
            i_local = i-myARXModel.n_in;
            % ## 1 #########
            for j = (1:myARXModel.n_in)
                j_local = j;

                if i_local ~= j_local
                    Jacobi(i,j) = ...
                        -3 * (i_local * pi)^2 ...
                        * (j_local * pi)^2 ...
                        * 2 * staticDeformation(j_local,sol.e) ...
                        * staticDeformation(i_local,sol.e);
                end
                if i_local == j_local
                    Jacobi(i,j) = ...
                        -(i_local * pi)^4 ...
                        -R0 * (i_local * pi)^2 ...
                        -3 * (i_local * pi)^2 ...
                        * (j_local * pi)^2 ...
                        * 3 * (staticDeformation(i_local,sol.e))^2;

                    for n = 1:myARXModel.n_in
                        if n ~= i_local
                            Jacobi(i,j) = Jacobi(i,j) ...
                                -3 * (i_local * pi)^2 ...
                                * (n * pi)^2 ...
                                * (staticDeformation(n,sol.e))^2 ;
                        end
                    end
                end

                Jacobi(i,j) = Jacobi(i,j) ...
                    + (-lambda*(l/h)*beta) * aero.D(i_local,j_local);
            end

            % ## 2 #########
            for j = (1:myARXModel.n_in) + myARXModel.n_in
                % j_local = j-n_in;
                % reamin 0
            end
            % ## 3 #########
            for j = (1:n_aero) + 2*myARXModel.n_in
                j_local = j-2*myARXModel.n_in;
                Jacobi(i,j) = (-lambda*(l/h)*beta) * aero.C(i_local,j_local);
            end
        end


        for i = (1:n_aero) + 2*myARXModel.n_in
            i_local = i-2*myARXModel.n_in;
            % ## 1 #########
            for j = (1:myARXModel.n_in)
                j_local = j;
                Jacobi(i,j) = aero.B(i_local,j_local);
            end
            % ## 2 #########
            for j = (1:myARXModel.n_in) + myARXModel.n_in
                % j_local = j-n_in;
                % reamin 0
            end
            % ## 3 #########
            for j = (1:n_aero) + 2*myARXModel.n_in
                j_local = j-2*myARXModel.n_in;
                Jacobi(i,j) = aero.A(i_local,j_local);
            end
        end


        %% Examine stability

        lambda_eig = eig(Jacobi);
        isUnstable = any( (real(lambda_eig) > 0) );


        %% Gather results

        fixPoints.stability(count_pc_target,count_lambda) = isUnstable;
        fixPoints.staticDeformation{count_pc_target,count_lambda} = sol.e;
        fixPoints.lambda(count_lambda) = lambda;
        fixPoints.Cpc(count_pc_target) = Cpc;


    end
end


%%

save(string(datetime(datetime,'Format','yyyyMMdd_HHmmss'))+".mat")


%% Tool functions

% ===========================

function q = staticDeformation(n, q_static)
q = q_static(n);
end















