

# Introduction to the file structure:
## database_myARXModel.mat
Core data of ARX model. The data is saved in MATLAB .mat format.

Data structure:
```
myARXModel.na       na of ARX model
myARXModel.nb       nb of ARX model
myARXModel.n_in     number of input 
myARXModel.n_out    number of output
myARXModel.THETA    Identification matrix of ARX, contains Ai and Bi.
```

The coefficient matrix Ai and Bi can be extracted by following code.

```
anchor = 1;

for i = 1:myARXModel.na
    myARXModel.A(1:myARXModel.n_out,1:myARXModel.n_out,i) = ...
        myARXModel.THETA(1:myARXModel.n_out, anchor:anchor+myARXModel.n_out-1);
    anchor = anchor + myARXModel.n_out;
end
for i = 1:myARXModel.nb
    myARXModel.B(1:myARXModel.n_out,1:myARXModel.n_in,i) = ...
        myARXModel.THETA(1:myARXModel.n_out, anchor:anchor+myARXModel.n_in-1);
    anchor = anchor + myARXModel.n_in;
end
```


## LS_analysis_multi_main.m
A program for linear stability analysis of fixed points. The necessary comments are included.
The program contains:

- Load ARX Model
- Calculation setting
- Get ARX coefficient matrix
- Get static ARX model coefficient matrix
- Solve fixed point position
- Get state space model coefficient matrix
- Convert to continuous-time
- Get Jacobian matrix
- Examine stability
- Gather results

## strucEqu.m
Function of structural equations. The input of the function is

```
 e                 state variable 
 Cpc               cavity pressure coefficient
 lambda            non-dimensional dynamic pressure
 staticARX         coefficient matrix of static ARX model
 force_initial     static initial force during training
 M                 Mach number
 l                 plate length
 h                 plate thickness
 R0                initial in-plane force (non-dimensional)
```
The output of the function is \dot e.
