function e_dot = strucEqu(e, Cpc, lambda, staticARX, force_initial, M, l, h, R0)


% e                 state variable 
% Cpc               cavity pressure coefficient
% lambda            non-dimensional dynamic pressure
% staticARX         coefficient matrix of static ARX model
% force_initial     static initial force during training
% M                 Mach number
% l                 plate length
% h                 plate thickness
% R0                initial in-plane force (non-dimensional)


n_e = length(e);
e_dot = zeros(n_e,1);

n_mode = 6;

beta_air = (M*M-1)^0.5;
gamma_air = 1.4;


f = staticARX * e(1:n_mode);

for n = 1:n_mode

    A = (n*pi)^4;

    B = R0 * (n*pi)^2;

    C = 0.0;
    for i = 1:n_mode
        C = C+(i*pi*e(i))^2;
    end
    C = C*3*(n*pi)^2;

    D = lambda*(l/h)*beta_air ...
        *(f(n)+force_initial(n) ...
        -2/(gamma_air*M^2) * ((1-cos(n*pi))/(n*pi)) * (Cpc-1));

    e_dot(n) = -(A+B+C)*e(n)-D;
end


end
